#pragma DEFJ(TIM1_INIT=0xFE)									//timer 1 jako prdko transmisji (19200bps)
																		//dla rezonatora 11,0592MHz TH1=0xFD; dla 7,3728MHz TH1 = 0xFE
#pragma SMALL														//wybr modelu pamici programu
#include "reg8252.h"												//definicje rejestrw
#include "stdio.h"												//funkcje wejcia - wyjcia

#define WORD unsigned int

//bit umoliwiajcy "wyczenie" realizowania polece przez terminal
bit ON = 1;

//definicje nagwkw funkcji programu
char in(char data *bufor);
char out(char data *bufor);
char status(char data *bufor);
char on(char data *bufor);
char off(char data *bufor);
char help(char data *bufor);


//definicja typu dla tablicy - wykazu polece
typedef struct
{
  char code *komenda;
  char (code *funkcja)(char data *);
}komendy;


//tablica z wykazem polece
code komendy wykaz[] = 											//wykaz komend i powizanych z nimi funkcji
{
	"IN", in,
	"OUT", out,
	"STATUS", status,
	"ON", on,
	"OFF", off,
	"HELP", help,
	"?", help,
	"", NULL       												//koniec wykazu
};


//odczyt portu o podanym numerze
char in(char data *bufor)
{
	int port;
	char odczyt;

	if (ON)
	{
		if(sscanf(bufor, "%i", &port ) != 1 ) 				//funkcja SCANF zwraca warto integer zawierajc numer portu
  			printf("%s\n", "Podaj numer portu!");			//jeli brak numeru, to komunikat o bdzie
  		else
  		{
	  		switch(port)
			{
	   		case 0:												//port 0
		    		odczyt = P0; 
			    	break;
	   		case 1:												//port 1
	    			odczyt = P1; 
	    			break;
		   	case 2:												//port 2
		    		odczyt = P2; 
	   	 		break;
	   		case 3:												//port 3
		    		odczyt = P3; 
		    		break;
	   		default:												//poza zakresem
	    			printf("%s\n", "Bledny numer portu!");
	    			return(0);
			}
			printf( "Port P%d = %02X (%d)\n", port, odczyt, odczyt );
		}
	}
	else
	{
		printf("%s\n","BLAD: Urzadzenie wylaczone");		//komunikat o wyczeniu
	}
	return 0;
}


char out(char data *bufor)
{
  WORD port, wartosc;
  
	if (ON)
	{
		if(sscanf(bufor, "%i%i", &port, &wartosc ) != 2 ) 
			printf("%s\n", "Bledne argumenty wywolania! Podaj: OUT <port> <wartosc>");
		else
		{
			switch(port)
  			{
				case 0: 												//port 0
					P0 = wartosc; 
					break;
				case 1: 												//port 1
					P1 = wartosc; 
					break;
				case 2: 												//port 2
					P2 = wartosc; 
					break;
				case 3: 												//port 3
					P3 = wartosc | RXD | TXD;
					break;
				default:
					printf("%s\n", "Bledny numer portu!");
			}
		}
	}
	else
	{
		printf("%s\n","BLAD: Urzadzenie wylaczone");		//komunikat o wyczeniu
	}
	return(0);
}


//podanie statusu "sterownika"
char status(char data *bufor)
{
	if (ON) printf("%s\n","STATUS = AKTYWNY"); else printf("%s\n","STATUS = WYLACZONY");
	return(0);
}


//zaczenie sterownika
char on(char data *bufor)
{
	ON = 1;
}


//wyczenie sterownika
char off(char data *bufor)
{
	ON = 0;
}


//opisy komend
char help(char data *bufor)
{
  char code *opis = 	
  			"Realizowane polecenia:\n"
			"in <numer portu> (odczyt portu o podanym numerze)\n"
			"out <numer portu> <wartosc> (zapis do portu o podanym numerze liczby WARTOSC)\n"
			"status (informacja o statusie WYL./AKT. \n"
			"on (zalaczenie przyjmowania nastaw) \n"
			"off (wylaczenie przyjmowania nastaw) \n"
			"help lub ? (informacja o realizowanych poleceniach)\n";
  printf( "%s", opis );
  return(0);
}


//wyszukiwanie komend oraz wywoanie odpowiadajacych im funkcji
char command(char data *bufor)
{
	char i, j;														//indeksy dla 256 komend o maks.dugoci 256 znakw
	
	for (i = 0;;)
		for (j = 0;; )
		{
			if(wykaz[i].komenda[j] != 0)						//jeli komenda w wykazie rna od znaku "pustego"
			{
				if(((wykaz[i].komenda[j]^bufor[j]) & 0x5F) == 0)	//odrzucamy kody ASCII maych liter(zamiana na due)
				{
          		j++;
		 			continue;										//nastpny znak
				}
				i++;
				break;                  						//nastpna komenda
		}
		if( j == 0 ) 
		{
			printf("%s\n","BLAD: Nie rozpoznano komendy!");	//bd - nie ma komendy w wykazie
			return(0);
		} else return (wykaz[i].funkcja(bufor+j));		//powrt pod adres - wskanik, to jest wykonanie funkcji spod
	}																	//wskazanego adresu
}



//program gwny
void main()
{
	char data bufor[21];

	PCON |= 0x80;													//ustawienie bitu prescalera zegara transmisji
	while(1)
	{
		putchar('>');												//gotowo do przyjcia komendy
		gets(bufor);												//pobranie komendy
		command(&bufor);											//wykonanie komendy
		printf("%s\n","OK");
	}
}

