#pragma DEFJ(TIM1_INIT=0xFE)								//timer 1 jako prdko transmisji (19200bps)
																	//dla rezonatora 11,0592MHz TH1=0xFD; dla 7,3728MHz TH1 = 0xFE
#pragma SMALL													//wybr modelu pamici programu
#include "reg8252.h"											//definicje rejestrw
#include "stdio.h"											//funkcje wejcia - wyjcia
#include "lcd4b.h"

#define WORD unsigned int

//bit umoliwiajcy "blokad" funkcji programu
bit ON = 1;

//definicje nagwkw funkcji realizujacych polecenia
char cursor(char data *bufor);
char clr(char data *bufor);
char gotoxy(char data *bufor);
char write(char data *bufor);
char cwrite(char data *bufor);
char fill(char data *bufor);
char define(char data *bufor);
char help(char data *bufor);
char on(char data *bufor);
char off(char data *bufor);
char status(char data *bufor);
char init(char data *bufor);


//definicja typu dla tablicy polece
typedef struct
{
  char code *komenda;										//wskanik do nazwy komendy w obszarze CODE
  char (code *funkcja)(char data *);					//typ dokonujcy konwersji nazwy funkcji na wskanik do tej funkcji
}komendy;


code komendy wykaz[] = 										//wykaz komend i powizanych z nimi funkcji
{
	"CURSOR", cursor,
	"CLR", clr,
	"GOTOXY", gotoxy,
	"WRITE", write,
	"CWRITE", cwrite,
	"FILL", fill,
	"DEF", define,
	"INIT", init,
	"ON", on,
	"OFF", off,
	"STATUS", status,
	"HELP", help,
	"?", help,
	"", NULL       											//koniec wykazu
};


//funkcja pomocnicza - wysanie przez UART komunikatu o wyczeniu funkcji programu
void blokada()                         				
{
	printf("%s\n","BLAD: Blokada dostepu");
}


//funkcja obsugi zaczenia - wyczenia kursora
char cursor(char data *bufor)
{
	WORD temp;
	
	if (ON)
	{
		if(sscanf(bufor, "%i", &temp ) != 1) printf("%s\n", "Blad! Podaj CURSOR <kod 0..2>");	//komunikat o bdzie
  		else switch (temp)
  		{
  			case 0:
  				WriteToLcdCtrlRegister(0x0C);
  				break;
  			case 1:
  				WriteToLcdCtrlRegister(0x0E);
  				break;
  			case 2:
  				WriteToLcdCtrlRegister(0x0F);
  				break;
  			default:
  				printf("%s\n","Parametr musi by z zakresu 0..2");
  		}
	}
	else
	{
		blokada();												//komunikat o wyczeniu
	}
	return 0;
}


//kasowanie ekranu LCD
char clr(char data *bufor)
{
	if (ON) LcdClrScr(); else blokada();
}


//pozycjonowanie kursora na ekranie LCD na wsprzdnych X, Y
char gotoxy(char data *bufor)
{
  WORD x, y;
  
	if (ON)
	{
		if(sscanf(bufor, "%i%i", &x, &y) != 2 ) printf("%s\n", "Blad! Podaj: GOTOXY <x> <y>");
		else LcdGotoXY(x, y);
	}
	else
	{
		blokada();												//komunikat o wyczeniu
	}
	return(0);
}


//wywietlenie tekstu podanego jako parametr wywoania
char write(char data *bufor)
{
	if (ON)
	{
		bufor++;
		while (*bufor != 0x0D && *bufor != 0) LcdWrite(*bufor++);
	}
	else
	{
		blokada();												//komunikat o wyczeniu
	}
	return(0);
}
	
	
//wywietlenie znaku o kodzie podanym jako parametr wywoania
char cwrite(char data *bufor)
{
	WORD x;

	if (ON)
	{
		if(sscanf(bufor, "%i", &x) != 1) printf("%s\n", "Blad! Podaj: CWRITE <kod>");
		else LcdWrite(x);
	}
	else
	{
		blokada();												//komunikat o wyczeniu
	}
	return(0);
}
	
	
//wypenienie ekranu LCD znakami o kodzie podanym jako parametr wywoania
char fill(char data *bufor)
{
	WORD x;
	char i;
	
	if (ON)
	{
		if(sscanf(bufor, "%i", &x) != 1) printf("%s\n", "Blad! Podaj: FILL <kod>");
		else 
		{
			LcdClrScr();
			for (i=0; i<81; i++) LcdWrite(x);
		}
	}
	else
	{
		blokada();												//komunikat o wyczeniu
	}
	return(0);
}


//definiowanie wasnych znakw
char define(char data *bufor)
{
	WORD x, by0, by1, by2, by3, by4, by5, by6, by7;
	
	if (ON)
	{
		if(sscanf(bufor, "%i%i%i%i%i%i%i%i%i", &x, &by0, &by1, &by2, &by3, &by4, &by5, &by6, &by7) != 9) 
			printf("%s\n", "Blad! Podaj: DEF <kod> <bajt0> <bajt1> <bajt2> <bajt3> <bajt4> <bajt5> <bajt6> <bajt7> ");
		else 
		{
			if (x < 10)
			{
				WriteToLcdCtrlRegister(0x40);				//ustawienie trybu definicji
				LcdWrite(by0);
				LcdWrite(by1);
				LcdWrite(by2);
				LcdWrite(by3);
				LcdWrite(by4);
				LcdWrite(by5);
				LcdWrite(by6);
				LcdWrite(by7);
				WriteToLcdCtrlRegister(0x80);				//przeczenie do trybu wywietlania
			}
			else
			{
				printf("%s\n", "Bledny kod znaku (0..9) lub polecenie!");
			}
		}
	}
	else
	{
		blokada();												//komunikat o wyczeniu
	}
	return(0);
}


//funkcja inicjalizacji wywietlacza
char init(char data *bufor)
{
	if (ON)
	{
		LcdInitialize();
		LcdClrScr();
	} else blokada();
}


//podanie statusu "sterownika"
char status(char data *bufor)
{
	if (ON) printf("%s\n","STATUS = AKTYWNY"); else printf("%s\n","STATUS = WYLACZONY");
	return(0);
}


//zaczenie sterownika
char on(char data *bufor)
{
	ON = 1;
}


//wyczenie sterownika
char off(char data *bufor)
{
	ON = 0;
}


//opisy komend
char help(char data *bufor)
{
  char code *opis = 	
  			"Realizowane polecenia:\n"
			"CURSOR <kod> (zmiana kursora,kod=0..2)\n"
			"CLR (kasowanie ekranu)\n"
			"GOTOXY <x> <y> (ustawienie kursora na x, y)\n"
			"WRITE <tekst> (wyswietlenie tekstu)\n"
			"CWRITE <kod> (wyswietlenie znaku o kodzie <kod>)\n"
			"FILL <kod> (wypelnienie LCD znakiem)\n"
			"DEF <kod> <bajt0> .. <bajt7> (definiowanie znaku)\n"
			"INIT (inicjalizacja wyswietlacza)\n"
			"STATUS (informacja o statusie WYL./AKT. \n"
			"ON (zalaczenie przyjmowania polecen) \n"
			"OFF (wylaczenie przyjmowania polecen) \n"
			"HELP lub ? (informacja o realizowanych poleceniach)\n";
  printf( "%s", opis );
  return(0);
}


//wyszukiwanie komend oraz podejmowanie akcji zalenie od wskanika w tablicy
char command(char data *bufor)
{
	char i, j;														//indeksy dla 256 komend o maks.dugoci 256 znakw
	
	for (i = 0;;)
		for (j = 0;; )
		{
			if(wykaz[i].komenda[j] != 0)						//jeli komenda w wykazie rna od znaku "pustego"
			{
				if(((wykaz[i].komenda[j]^bufor[j]) & 0x5F) == 0)	//odrzucamy kody ASCII maych liter(zamiana na due)
				{
          		j++;
		 			continue;										//nastpny znak
				}
				i++;
				break;                  						//nastpna komensa
		}
		if( j == 0 ) 
		{
			printf("%s\n","BLAD: Nie rozpoznano komendy!");	//bd - nie ma komendy w wykazie
			return(0);
		}
		else return (wykaz[i].funkcja(bufor+j));			//powrt pod adres - wskanik, to jest wykonanie funkcji spod
	}																	//wskazanego adresu
}



//program gwny
void main()
{
	char data bufor[41];

	LcdInitialize();												//inicjalizacja wywietlacza
	LcdClrScr();
	
	PCON |= 0x80;													//ustawienie bitu prescalera zegara transmisji
	while(1)
	{
		putchar('>');												//gotowo do przyjcia komendy
		gets(bufor);												//pobranie komendy
		command(&bufor);											//wykonanie komendy
		printf("%s\n","OK");
	}
}

