//biblioteka funkcji wywietlacza lcd w trybie 4-bity
//-----------------------------------------------------

#include "LCD4B.h"

//bity sterujce LCD
sbit LcdEnable = 		ENABLE;
sbit LcdRead = 		READ;
sbit LcdReg = 			REGISTER;


// opnienie okoo 1 milisekundy dla kwarcu 7,3728MHz
void Delay (unsigned int k) 	
{
	unsigned int i,j;
	for (j = 0; j < k; j++)
		for (i = 0; i <= 296; i++);
}


// zapis bajtu do lcd
void WriteByteToLcd(char X)
{
	PORT |= 0xF0;							// ustawienie grnej powki portu P2 na "1"
	PORT &= (X | 0x0F);					// "bezkolizyjny" zapis 1-szej powki bajtu (przez funkcj logiczn)
	LcdEnable = 0;							// zapis do wywietlacza (opadajce zbocze sygnau E)
	
	LcdEnable = 1;							// zapis 2-giej powki bajtu
	X <<= 4;									// przesunicie 4x w lewo
	PORT |= 0xF0;							// ustawienie grnej powki portu P2 na "1"
	PORT &= (X | 0x0F);					// zapis 2-giej powki bajtu
	LcdEnable = 0;							// opadajce zbocze E - zapis do LCD

	Delay(1);
}

// zapis bajtu do rejestru kontrolnego LCD
void WriteToLcdCtrlRegister(char X)
{
	LcdReg = 0;								// ustawienie sygnaw sterujcych
	LcdRead = 0;
	LcdEnable = 1;
	WriteByteToLcd(X);
}


// write a unsigned character to lcd screen
void LcdWrite(char X)
{
	LcdReg = 1;
	LcdRead = 0;
	LcdEnable = 1;
	WriteByteToLcd(X);
}


//czyszczenie ekranu LCD
void LcdClrScr(void)
{
	WriteToLcdCtrlRegister(0x01);
}


//inicjalizacja wywietlacza LCD w trybie 4 bity
void LcdInitialize(void)
{
	char i;
	
	Delay(15);
	LcdReg = LcdEnable = LcdRead = 0;// wyzerowanie linii LcdReg,LcdRead,LcdEnable
	for (i = 0; i<3; i++)
	{
		LcdEnable = 1;						// impuls na E
		PORT &= 0x3F;						// ustawienie wart.inicjujcej
		LcdEnable = 0;
		Delay(5);
	}
	LcdEnable = 1;							// wpisanie wartoci 2 do rej.kontr.
	PORT &= 0x2F;							// tylko "grne" 4 bity
	LcdEnable = 0;
	Delay(1);
	WriteToLcdCtrlRegister(0x28);		// interfejs 4 bity,znaki 5x7
	WriteToLcdCtrlRegister(0x08);		// wyczenie LCD
	WriteToLcdCtrlRegister(0x01);		// kasowanie ekranu,powrt do home
	WriteToLcdCtrlRegister(0x06);		// przesuwanie kursora z inkrement.
	WriteToLcdCtrlRegister(0x0C);		// zaczenie wywietlacza
}

	
// ustawia kursor na wsprzdnych x,y
void GotoXY(char x, char y)
{
	switch (y)								// obliczenie o ile naley zwikszy x w zalenoci od wartoci y 
	{
		case 0:
			x += 0x80;						// x = x + 0x80
			break;
		case 1:
			x += 0xC0;						// x = x + 0xC0
			break;
		case 2:
			x += 0x94;						// x = x + 0x94
			break;
		case 3:
			x += 0xD4;						// x = x + 0xD4
			break;
	}
	WriteToLcdCtrlRegister(x);
}


// wywietla tekst na wsprzdnych x, y
void WriteTextXY(char x, char y, char *S)
{
	while (*S)								// ptla dziaa dotd,a napotkany zostanie znak koca acuch (/0)
	{
		GotoXY(x, y);						// wyliczenie adresu dla znaku
		LcdWrite(*S);						// wywietlenie pojedynczego znaku
		x++; 									// nastpna pozycja x na ekranie
		S++;									// nastpna pozycja wskanika,tzn.nastpny znak napisu
		if (x > 19)							// jeli x>19 to nastpna linia
		{
			x = 0; 
			y++;
		}
	}
}


// wywietla tekst na wsprzdnych x, y
void WriteText(char *S)
{
	while (*S)								// ptla dziaa dotd,a napotkany zostanie znak koca acuch (/0)
	{
		LcdWrite(*S);						// wywietlenie pojedynczego znaku
		S++;
	}
}


// definiowanie znakw z tablicy CGRom
void DefineSpecialCharacters(char *ptr)
{
	WriteToLcdCtrlRegister(0x40);		// ustawienie trybu definicji
	while (*ptr != 0)						// ptla wykonywana do napotkania znaku koca tablicy
	{
		LcdWrite(*ptr);					// zapis znaku do lcd cgram
		ptr++;								// nastpna pozycja tablicy (wskanika)
	}
	WriteToLcdCtrlRegister(0x80);		// przeczenie do trybu wywietlania
}

 
