/************************************************
 Obsuga wywietlacza LCD w trybie 4 bity
 RC-51 Raisonance
*************************************************/
// wybr modelu pamici	
#pragma SMALL
// doczenie definicji rejestrw '51
#include <reg51.h>

// definicje znakw specjalnych dla wywietlacza LCD
char code CGRom[65] = {	0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,		// "kratka"			0x00
								0xC0,0xC0,0xFF,0xF1,0xF1,0xF1,0xFF,0xC0,		// pusty kwadrat	0x01
							  	0xC0,0xC0,0xFF,0xFF,0xFF,0xFF,0xFF,0xC0,		// kwadrat zacz.	0x02
							  	0xE0,0xFF,0xFF,0xFF,0xE0,0xE0,0xE0,0xE0,		// linia				0x03
							  	0xFF,0xFF,0xF9,0xF3,0xE7,0xF3,0xF9,0xFF,		// znak w lewo		0x04
							  	0xFF,0xFF,0xF3,0xF9,0xFC,0xF9,0xF3,0xFF,		// znak w prawo	0x05
				           	0xFF,0xFF,0xFB,0xF1,0xE4,0xEE,0xFF,0xFF,		// znak w gr		0x06
				           	0xFF,0xFF,0xFF,0xEE,0xE4,0xF1,0xFB,0xFF,		// znak w d		0x07
				           	0x00};

// port,do ktrego podczono wywietlacz LCD
#define	PORT			P2
// 4 najstarsze bity, to bity danych, 4 modsze mog by w dowolny sposb skonfigurowane jako sterujce
// po zmianie definicji mog to by rwnie 2 rne porty,jednak w programie przykadowym zakadam 
// wykorzystanie tylko jednego z portw mikrokontrolera

// bity sterujce LCD
sbit LcdEnable = 		PORT^0;
sbit LcdRead = 		PORT^3;
sbit LcdReg = 			PORT^1;


// opnienie okoo 1 milisekundy dla kwarcu 7,3728MHz
void Delay (unsigned int k) 	
{
	unsigned int i,j;
	for (j = 0; j < k; j++)
		for (i = 0; i <= 296; i++);
}


// zapis bajtu do lcd,osobna procedura dla zmniejszenia objtoci kodu wynikowego
void WriteByteToLcd(char X)
{
	LcdEnable = 1;
	PORT |= 0xF0;							// ustawienie grnej powki portu PORT na "1"
	PORT &= (X | 0x0F);					// "bezkolizyjny" zapis 1-szej powki bajtu (przez funkcj logiczn)
	LcdEnable = 0;							// zapis do wywietlacza (opadajce zbocze sygnau E)
	LcdEnable = 1;							// zapis 2-giej powki bajtu
	X <<= 4;									// przesunicie 4x w lewo
	PORT |= 0x0F;							// maskowanie 4 modszych bitw
	PORT |= (X | 0xF0);					// ustawienie grnej powki portu PORT na "1"
												// maskowanie 4 modszych bitw
	LcdEnable = 0;							// opadajce zbocze E - zapis do LCD
	Delay(1);								// czas na przetwarzanie dla LCD
}

// zapis bajtu do rejestru kontrolnego LCD
void WriteToLcdCtrlRegister(char X)
{
	LcdReg = LcdRead = 0;				// ustawienie sygnaw sterujcych
	WriteByteToLcd(X);
}


// zapis bajtu do pamici obrazu
void LcdWrite(char X)
{
	LcdReg = 1;								// ustawienie sygnaw sterujcych
	LcdRead = 0;
	LcdEnable = 1;
	WriteByteToLcd(X);
}


// czyszczenie ekranu LCD
void LcdClrScr(void)
{
	WriteToLcdCtrlRegister(0x01);
}


// inicjalizacja wywietlacza LCD w trybie 4 bity
void LcdInitialize(void)
{
	char i;
	
	Delay(15);
	P0 = 0x0F;								// wyzerowanie linii LcdReg,LcdRead,LcdEnable
	for (i = 0; i<3; i++)
	{
		LcdEnable = 1;						// impuls na E
		PORT &= 0x3F;						// ustawienie wart.inicjujcej
		LcdEnable = 0;
		Delay(5);
	}
	LcdEnable = 1;							// wpisanie wartoci 2 do rej.kontr.
	PORT &= 0x2F;							// tylko "grne" 4 bity
	LcdEnable = 0;
	Delay(1);
	WriteToLcdCtrlRegister(0x28);		// interfejs 4 bity,znaki 5x7
	WriteToLcdCtrlRegister(0x08);		// wyczenie LCD
	WriteToLcdCtrlRegister(0x01);		// kasowanie ekranu,powrt do home
	WriteToLcdCtrlRegister(0x06);		// przesuwanie kursora z inkrement.
	WriteToLcdCtrlRegister(0x0C);		// zaczenie wywietlacza
}

	
// ustawia kursor na wsprzdnych x,y
void GotoXY(char x, char y)
{
	switch (y)								// obliczenie o ile naley zwikszy x w zalenoci od wartoci y 
	{
		case 0:
			x += 0x80;						// x = x + 0x80
			break;
		case 1:
			x += 0xC0;						// x = x + 0xC0
			break;
		case 2:
			x += 0x94;						// x = x + 0x94
			break;
		case 3:
			x += 0xD4;						// x = x + 0xD4
			break;
	}
	WriteToLcdCtrlRegister(x);
}


// wywietla tekst na wsprzdnych x, y
void WriteTextXY(char x, char y, char *S)
{
	while (*S)								// ptla dziaa dotd,a napotkany zostanie znak koca acuch (/0)
	{
		GotoXY(x, y);						// wyliczenie adresu dla znaku
		LcdWrite(*S);						// wywietlenie pojedynczego znaku
		x++; 									// nastpna pozycja x na ekranie
		S++;									// nastpna pozycja wskanika,tzn.nastpny znak napisu
		if (x > 19)							// jeli x>19 to nastpna linia
		{
			x = 0; 
			y++;
		}
	}
}


// wywietla tekst na wsprzdnych x, y
void WriteText(char *S)
{
	while (*S)								// ptla dziaa dotd,a napotkany zostanie znak koca acuch (/0)
	{
		LcdWrite(*S);						// wywietlenie pojedynczego znaku
		S++;
	}
}


// definiowanie znakw z tablicy CGRom
void DefineSpecialCharacters(char *ptr)
{
	WriteToLcdCtrlRegister(0x40);		// ustawienie trybu definicji
	while (*ptr != 0)						// ptla wykonywana do napotkania znaku koca tablicy
	{
		LcdWrite(*ptr);					// zapis znaku do lcd cgram
		ptr++;								// nastpna pozycja tablicy (wskanika)
	}
	WriteToLcdCtrlRegister(0x80);		// przeczenie do trybu wywietlania
}

// program gwny
void main(void)
{
	char ix = 1, iy = 1, x, y, i = 0;
	
	LcdInitialize();
	DefineSpecialCharacters(&CGRom);
	while (1)
	{
		WriteTextXY(x, y, " ");
		if (ix == 1) x++; else x--;
		if (iy == 1) y++; else y--;
		if (x == 19) ix = 0;
		if (x == 0) ix = 1;
		if (y == 3) iy = 0;
		if (y == 0) iy = 1;
		WriteTextXY(x, y, 0x01);
		Delay(50);
	}
}

