/***********************************************
 demonstracja budowy menu wielopoziomowego
 -----------------------------------------------
 klawiatura matrycowa 4 (wiersze) x 3 (kolumny)
 wywietlacz lcd 4 (linie) x 20 (znakw)
 mikrokontroler AT89S8252 / 7,3728MHz
 ***********************************************/
#pragma SMALL
#include "REG8252.h"
#include "LCD4B.h"
#include "KBD3x4.h"

// definicje znakw specjalnych dla wywietlacza LCD
char code CGRom[65] = {	0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,		// "kratka"			0x00
								0xC0,0xC0,0xFF,0xF1,0xF1,0xF1,0xFF,0xC0,		// pusty kwadrat	0x01
							  	0xC0,0xC0,0xFF,0xFF,0xFF,0xFF,0xFF,0xC0,		// kwadrat zacz.	0x02
							  	0xE0,0xFF,0xFF,0xFF,0xE0,0xE0,0xE0,0xE0,		// linia				0x03
							  	0xFF,0xFF,0xF9,0xF3,0xE7,0xF3,0xF9,0xFF,		// znak w lewo		0x04
							  	0xFF,0xFF,0xF3,0xF9,0xFC,0xF9,0xF3,0xFF,		// znak w prawo	0x05
				           	0xFF,0xFF,0xFB,0xF1,0xE4,0xEE,0xFF,0xFF,		// znak w gr		0x06
				           	0xFF,0xFF,0xFF,0xEE,0xE4,0xF1,0xFB,0xFF,		// znak w d		0x07
				           	0x00};

//typy i stae uywane przez menu
//uwaga: linie menu maj maksymaln dugo 18 znakw
//ze wzgldu na to, e czasami jedna linia jest wywietlana na miejscu drugiej
//powinny one mie jednakow (np.uzupenion spacjami) dugo
#define maxoptlen 18 
#define maxlines 4

//definicja dla pozycji pod-menu
typedef struct
{
	char name[maxoptlen];						//napisy menu
	char ptr;										//liczba,ktr naley doda do adresu tablicy
	char y0;											//pooenie znacznika wyboru w poziomie
	char retval;									//warto zwracana w wyniku wyboru
} submenuitem;

//definicja dla pozycji menu gwnego
typedef struct
{
	char name[maxoptlen];						//napisy menu
	char ptr;										//liczba,ktr naley doda do adresu tablicy
	char y0;											//pooenie znacznika wyboru w poziomie
	code submenuitem *nextmenu;				//wskanik do tablicy z wykazem opcji podmenu
	char retval;									//warto zwracana w wyniku wyboru
} menuitem;

//wykaz opcji menu wraz z pooeniem znacznika, pozycj wskanika oraz adresem wykazu pod-menu
code submenuitem menu1[4] = {					//pod-menu 1
{"podmenu 1.1 _ _",0,0,11},{"podmenu 1.2 _  ",0,1,12},{"podmenu 1.3 _ _",0,2,13},
{0,0,0,0}};

code submenuitem menu2[8] = {					//pod-menu 2
{"podmenu 2.1 _ _",0,0,21},{"podmenu 2.2 _  ",0,1,22},{"podmenu 2.3 _ _",0,2,23},
{"podmenu 2.4 _  ",0,3,24},{"podmenu 2.5 _ _",1,3,25},{"podmenu 2.6 _  ",2,3,26},
{"podmenu 2.7 _ _",3,3,27},
{0,0,0,0}};

code submenuitem menu4[3] = {					//pod-menu 3
{"podmenu 4.1 _ _",0,0,41},{"podmenu 4.2 _  ",0,1,42},
{0,0,0,0}};

code submenuitem menu5[4] = {					//pod-menu 4
{"podmenu 5.1 _ _",0,0,51},{"podmenu 5.2 _  ",0,1,52},{"podmenu 5.3 _ _",0,2,53},
{0,0,0,0}};

code menuitem mainmenus[6] = {				//menu gwne
{"opcja menu g.1 . ",0,0,&menu1,0},{"opcja menu g.2 ..",0,1,&menu2,0},{"opcja menu g.3 . ",0,2,0,3},
{"opcja menu g.4 ..",0,3,&menu4,0},{"opcja menu g.5 . ",1,3,&menu5,0},
{0,0,0,0,0}};

//aktualna pozycja w menu gwnym
signed char POS_IN_MAINMENU;
//aktualna pozycja w pod-menu
signed char POS_IN_SUBMENU;

//usunicie znacznikw wyboru linii menu
void Clear_Marks(char y)
{
	GotoXY(0,y);
	LcdWrite(' ');
	GotoXY(19,y);
	LcdWrite(' ');
}

//wywietlenie menu w/g wykazu (od pozycji wskanika PTRMENUS liczby linii MAXLINES)
void Display_Sub_Menu(code submenuitem *ptrtable, char num_of_options)
{
	char temp;
	code submenuitem *ptrtemp;
	
	ptrtemp = (ptrtable + POS_IN_SUBMENU);											//ustawienie wskanika na pozycji,od ktrej wywietlane jest menu
	ptrtable += (ptrtemp->ptr);														//offset wskanika jest jednym z parametrw w wykazie menu
	for (temp = 0; (temp < maxlines && temp <= num_of_options); ++temp) WriteTextXY(1, temp, (ptrtable+temp)->name);
	GotoXY(0,ptrtemp->y0);																//wywietlenie lewego znacznika (kod 5)
	LcdWrite(0x05);
	GotoXY(19,ptrtemp->y0);																//wywietlenie prawego znacznika (kod 4)
	LcdWrite(0x04);
}

//obsuga pod-menu; jako parametr wywoania podawany jest adres tablicy z wykazem menu
char Sub_Menu(code submenuitem *ptrtable)
{
	char maxsubopt = 0;																	//maksymalna (wyliczana) liczba opcji podmenu
	code submenuitem *ptrtemp;                                           //wskanik pomocniczy

	if (ptrtable->name)																	//wejcie do obsugi,jeli nazwa pod-menu nie jest pusta
	{
		while ((ptrtable+maxsubopt)->name[0]) ++maxsubopt;						//wyznaczenie liczby opcji submenu
		LcdClrScr();																		//czyszczenie ekranu
		POS_IN_SUBMENU = 0;																//pozycja pocztkowa w pod-menu
		Display_Sub_Menu(ptrtable,maxsubopt);										//wywietlenie pod-menu
		ptrtemp = ptrtable;
		while (1)																			//ptla nieskoczona-wyjcie po wybraniu opcji menu
		{
			switch (Key_Number())														//odczyt klawiatury
			{     
				case 0x02:																	//klawisz W GR
					Clear_Marks(ptrtemp->y0);											//kasowanie poprzednich znacznikw wyboru
					if (--POS_IN_SUBMENU < 0) POS_IN_SUBMENU = maxsubopt-1;	//wyznaczenie pozycji w pod-menu i porwnanie z maksymaln liczb opcji
					ptrtemp = ptrtable+POS_IN_SUBMENU;								//wyliczenie wskazania do wybranej pozycji pod-menu
					Display_Sub_Menu(ptrtable,maxsubopt);							//wywietlenie pod-menu
					break;
				case 0x08:																	//klawisz W Dӣ
					Clear_Marks(ptrtemp->y0);											//kasowanie poprzednich znacznikw wyboru
					if (++POS_IN_SUBMENU >= maxsubopt)								//wyznaczenie pozycji w pod-menu i porwnanie z minimaln liczb opcji
					{
						POS_IN_SUBMENU = 0;
						ptrtemp = ptrtable;
					}
					ptrtemp = ptrtable+POS_IN_SUBMENU;								//wyliczenie wskazania do wybranej pozycji pod-menu
					Display_Sub_Menu(ptrtable,maxsubopt);							//wywietlenie pod-menu
					break;
				case 0x0C:																	//klawisz ENTER
					return(ptrtemp->retval);
				case 0x0B:																	//klawisz ESC
					return(0);
			}
		}
	} else return(0);
}

//wywietlenie menu w/g wykazu (od pozycji wskanika PTRMENUS liczby linii MAXLINES)
void Display_Main_Menu(code menuitem *ptrtable, char num_of_options)
{
	char temp;
	code menuitem *ptrtemp;
	
	ptrtemp = (ptrtable + POS_IN_MAINMENU);										//ustawienie wskanika na pozycji,od ktrej wywietlane jest menu
	ptrtable += (ptrtemp->ptr);														//offset wskanika jest jednym z parametrw w wykazie menu
	for (temp = 0; (temp < maxlines && temp <= num_of_options); ++temp) WriteTextXY(1, temp, (ptrtable+temp)->name);
	GotoXY(0,ptrtemp->y0);																//wywietlenie lewego znacznika (kod 5)
	LcdWrite(0x05);
	GotoXY(19,ptrtemp->y0);																//wywietlenie prawego znacznika (kod 4)
	LcdWrite(0x04);
}

//funkcja wywietajca menu,zwracany jest bajt kodu wyboru
char Main_Menu(code menuitem *ptrmenus, char maxoptions)
{
	code menuitem *ptrtemp;
	char temp;
	
	POS_IN_MAINMENU = 0;																	//wywietlenie menu gwnego
	Display_Main_Menu(ptrmenus,maxoptions);
	while (1)																				//ptla nieskoczona-wyjcie po wybraniu opcji menu
	{
		switch (Key_Number())															//odczyt klawiatury
		{     
			case 0x02:																		//klawisz W GR
				Clear_Marks((ptrmenus+POS_IN_MAINMENU)->y0);						//usunicie poprzedniego wskazania menu
				if (--POS_IN_MAINMENU < 0) POS_IN_MAINMENU = maxoptions-1;	//wyliczenie pozycji w menu gwnym,porwnanie z maks.liczb opcji
				Display_Main_Menu(ptrmenus,maxoptions);							//wywietlenie menu gwnego
				break;
			case 0x08:																		//klawisz W Dӣ
				Clear_Marks((ptrmenus+POS_IN_MAINMENU)->y0);						//usunicie poprzedniego wskazania menu
				if (++POS_IN_MAINMENU >= maxoptions) POS_IN_MAINMENU = 0;	//wyliczenie poz.w menu,porwnanie z minimaln liczb opcji
				Display_Main_Menu(ptrmenus,maxoptions);							//wywietlenie menu gwnego
				break;
			case 0x0C:																		//klawisz ENTER
				ptrtemp = ptrmenus;														//wyliczenie wskazania do pozycji w wykazie odpowiadajcej
				ptrtemp += POS_IN_MAINMENU;											//wyborowi
				if (ptrtemp->nextmenu == 0)											//jeli nie ustawiono wartoci NEXTMENU,to wejcie do obsugi
				{																				//pod-menu
					temp = Sub_Menu(ptrtemp->nextmenu);
					if (temp)																//koniec pracy,jeli zwracana warto jest rna od 0
						return(temp);
					else 
					{
						LcdClrScr();														//nacinito ESC-powrt do obsugi pod-menu
						Display_Main_Menu(ptrmenus,maxoptions);
					}
				}
				else
					return(ptrtemp->retval);											//funkcja po naciniciu ENTER zwraca jako rezultat swojego 
				break;																		//dziaania warto zapamitan w tablicy-wykazie
		}
	}
}

//program gwny
void main(void)
{
	LcdInitialize();
	DefineSpecialCharacters(&CGRom);
	LcdClrScr();
	while(1)
	{
		LcdClrScr();
		switch (Main_Menu(&mainmenus, 5))
		{
			case 11:		//<-- pod-menu 1.1
			case 12:		//1.2
			case 13:		//1.3
			case 21:		//2.1
			case 22:		//2.2
			case 23:		//2.3
			case 24:		//2.4
			case 25:		//2.5
			case 26:		//2.6
			case 27:		//2.7
			case 3:		//<-- menu gwne, opcja 3
			case 41:		//4.1
			case 42:		//4.2
			case 51:		//5.1
			case 52:		//5.2
			case 53:		//5.3
				break;
		}
	}
}

