/**********************************************************
 Dekoder kodu RC5 odbieranego z toru podczerwieni
 na podstawie materiaw zamieszczonych na stronie:
 http://www.ustr.net/infrared/infrared1.shtml
 ----------
 UWAGI n/t wykonania:
 Wejcie odbiornika podczerwieni podczone jest do wejcia
 przerwania INT0 (P3.2). Dekoder uywa Timer'a 0 do pomiaru
 czasu trwania bitw. Aplikacja wykonana bya w oparciu o
 mikrokontroler AT89C2051 z rezonatorem kwarcowym 12MHz.
***********************************************************/ 

#include  "reg51.h"


#define IE0_VECTOR 1					//numer wektora przerwania INT0
#define TF0_VECTOR 2					//numer wektora przerwania Timer'a 0

#define BYTE unsigned char   
#define WORD unsigned int
#define ENABLE 1
#define DISABLE 0

#define HighB(value) ((value >> 8) & 0xFF)
#define LowB(value)  (value & 0xFF)

#define RELOAD        -440			//czas RELOAD dobrany dla nadajnika EasyTip1 
#define FIRST_RELOAD  -870			//Code718 i rezonatora 12 MHZ 

#define _TOGGLE      0x80
#define _START2      0x40
#define _START1      0x20      

sbit clockline = P3^7;				//linia zegara zapisu do ukadw zewntrznych

typedef struct _BYTE
{
	BYTE b3;								//bardziej znaczcy bajt
	BYTE b2;
	BYTE b1;
	BYTE b0;								//mniej znaczcy bajt
}BYTES;

typedef union _LONGINT
{
	unsigned long l;					//unia umoliwiajca szybk konwersj
	BYTES  b;							//zmiennej typu longint na pojedyncze
}LONGUNION;								//bajty i odwrotnie (bajty na longint)


//zmienne globalne
bit	bRC5valid;						//bit znacznika ustawiany po odberaniu
											//1-go kodu RC5

BYTE  command;							//odebrana komenda
BYTE  subaddress;						//odebrany adres urzdzenia

BYTE  slopecount;						//licznik pomocniczy
LONGUNION shiftreg;					//rejestr przesuwny dla odebranego sowa RC5


/****************************************************************
 Obsuga przerwania od odbiornika RC5. Rozpoczyna ona
 pomiary odbieranego sygnau przez Timer 0
 ****************************************************************/
void RC5_StartIrq(void) interrupt IE0_VECTOR
{
   EX0 = DISABLE;						//wyczenie przerwa od INT0
   slopecount = 27;					//odbierane jest 27 prbek
   TH0 = HighB(RELOAD);				//nastawa Timer'a 0
   TL0 = LowB(RELOAD);
   TR0 = ENABLE;						//uruchomienie Timer'a 0 w celu pomiaru
   TF0 = 0;								//czasu trwania bitu
   ET0 = ENABLE;						//uruchomienie obsugi przerwania Timer'a 0
}


/**************************************************************
 Obsuga przerwania Timer'a 0. Po odebraniu 27 par bitw 
 przesuwajcy zawiera warto RC5 a stan bitu bRC5Valid zmieniany
 jest na "1"
 **************************************************************/
void TimingIrq(void) interrupt TF0_VECTOR using 1
{
	bit bLevel;

   bLevel = P3^2;						//testowanie stanu wejcia P3.2
   while (slopecount--)				//do odebrania 27 prbek
   {
   	TR0 = DISABLE;					//zatrzymanie Timer'a 0
		TH0 = HighB(FIRST_RELOAD);	//zaadowanie nowej wartoci dla reload
		TL0 = LowB (FIRST_RELOAD);
		TR0 = ENABLE;					//uruchomienie Timer'a 0
		if (bLevel)						//dodanie odczytanego poziomu do wartoci
		{									//rejestru przesuwnego
			shiftreg.b.b0 = shiftreg.b.b0 | 1; 
		}
		shiftreg.l = shiftreg.l << 1;
		return;       
	}
	TR0 = DISABLE;         			//zatrzymanie Timer'a 0
	ET0 = DISABLE;         			//wyczenie przerwa od Timer'a 0
	bRC5valid = 1;	      			//ustawienie bitu sygnalizacji, e
											//bufor zawiera odebran transmisj RC5
}


/*******************************************************
 Dekodowanie odebranej wartoci (wydzielenie adresu i
 komendy). Funkcja zwraca "1" jeli dekodowanie przebiego
 bezbdnie, inaczej zwracane jest "0"
 *******************************************************/
bit DecodeRC5Code(void)
{
	BYTE i, b;

	command = subaddress = 0;		//zerowanie adresu urzdzenia i kodu komendy
	shiftreg.l = shiftreg.l >> 1;

	for (i=0; i < 6 ; i ++)			//najpierw dekodowanie 6 bitw komendy
	{  
		b = shiftreg.b.b0 & 0x03;
		if(b == 0x02) command |= 0x40; 
			else if(b == 0x01) command &= ~0x40; 
				else return(0);
		command = command >> 1;
		shiftreg.l = shiftreg.l >> 2;
	}

	for (i=0;i < 5;i++)				//nastpnie dekodowanie 5 bitw adresu
	{
		b = shiftreg.b.b0 & 0x03;
		if(b == 0x02)subaddress = subaddress | 0x20;
			else if(b == 0x01) subaddress &= ~0x20;
				else return(0);  
		subaddress = subaddress >> 1;
		shiftreg.l = shiftreg.l >> 2;
   }

	b = shiftreg.b.b0 & 0x03; 
   if(b == 0x02)command |= _TOGGLE;
   	else if(b == 0x01)command &= ~_TOGGLE;
   		else return(0);  

   shiftreg.l = shiftreg.l >> 2;
   b = shiftreg.b.b0 & 0x03;			//bit synchronizacji numer 2 inwersja
   if(b==0x02) command &= ~_START2;	//dla 7-bitowych komend (extended RC5)
   	else if(b == 0x01) command |= _START2;
			else return(0);
  
	shiftreg.l = shiftreg.l >> 2;
	b=shiftreg.b.b0 & 0x03;				//bit synchronizacji numer 1
	if(b == 0x02) subaddress |= _START1;
		else if(b==0x01) subaddress = subaddress & ~_START1;
			else return(0);  

   return(1);								//sygnalizacja, e odbir poprawny
}


/*******************************************************
 Program gwny jako przykad aplikacji wysyajcej
 odebran komend i adres przez port P0
 *******************************************************/

//impuls zapisujacy dane w ukadzie zewntrznym
//(opadajce zbocze sygnau)
void Clock()
{
	clockline = 0;
	clockline = 0;
	clockline = 1;
}


void main(void)
{
	clockline = 1;							//stan spoczynkowy sygnau zegarowego
   TMOD = 0x11;							//nastawa trybu pracy Timer'w 0 i 1
												//oba pracuj w trybie Timer 16-bitowy                                                          
	EX0 = ENABLE;							//zaczenie przerwa INT0 wyzywalanych
	IE0 = 0;									//opadajcym zboczem sygnau
	IT0 = ENABLE;

	shiftreg.l = 0x02;					//inicjowanie wartoci bufora odbioru
	EA = ENABLE;							//zaczenie przerwa

	while (1)
   {
		if (bRC5valid)						//jeli odebrano poprawn transmisj RC5
		{
			bRC5valid=0;
			if (DecodeRC5Code())			//testowanie bufora odbioru, dekodowanie 
			{									//adresu i komendy
				P1 = subaddress;
				Clock();
				P1 = command;
				Clock();
			} 
			shiftreg.l=0x02;				//restart dekodera
			IE0 = 0;
			EX0 = ENABLE;        
		}
	}
}
 
