/*****************************************************************
 Termometr radiowy zbudowany z wykorzystaniem ukadu AT89C2051 
 oraz moduw RR i RT firmy Soyter Elektronik
 
 Mikrokontroler wprowadzany jest w tryb IDLE, z ktrego wybudzany 
 jest przez przerwanie Timera 0; wwczas to nastpuje pomiar i 
 przesanie wyniku przez port szeregowy
******************************************************************/

#pragma TINY								//wybr modelu pamici: to rozmiar kodu
												//wynikowego nie moe przekroczy 2kB
												
#pragma DEFJ(TIM1_INIT=0xF8)			//timer 1 ustala prdko transmisji
												//tutaj 2400 bodw (SMOD bdzie rwny "0")

#include <reg51.h>						//doczenie definicji rejestrw uK
#include <stdio.h>						//biblioteka funkcji obsugi UART
#include "1wire.h"						//biblioteka funkcji obsugi ukadw z
												//interfejsem 1WIRE

//definicje staych symbolicznych:
#define ENABLE		1						//dla bitw rejestrw kontrolnych
#define DISABLE	0
#define ON	0								//dla diody LED
#define OFF	1

//definicje linii doczenia kluczy wczajcych zasilanie
//diody LED i nadajnika
sbit TRXPOWER = P1^0;					//wyjcie sterujce zaczaniem zasilania nadajnika
sbit LEDDIODE = P1^7;					//wyjcie sterujce zaczaniem diody LED


//program gwny
void main (void)
{
	TMOD &= 0xF1;							//timer 0 jako 16-bitowy timer, Timer 1 bez zmian
	TL0 = TH0 = 0;							//inicjowanie wartoci rejestrw Timer'a 0
	TRXPOWER = LEDDIODE = OFF;			//wyczenie diody LED i zasilania nadajnika globalne 
	EA = ET0 = ENABLE;					//zezwolenie na przyjmowanie przerwa oraz zaczenie
												//przerwa od Timer'a 0
	TR0 = ENABLE;							//uruchomienie Timer'a 0
	for (;;) PCON |= 0x01;				//ptla nieskoczona, w ktrej CPU wprowadzone w tryb IDLE 
												//oczekuje na przerwania od Timer'a 0
}

//opnienie okoo 1 milisekundy dla kwarcu 7,3728MHz
void delay(unsigned int time)
{
	unsigned char j;
	
	while (time-- >= 1)					//wykonanie ptli for zajmuje okoo 1 msek.
		for (j=0; j<65; j++);			//ptla jest powtarzana "time" razy
}

//odczyt danych z DS1820 po konwersji
signed int odczyt_DS1820 (void)
{
	char tl, th;
	
	PCON &= 0x7F;							//ustawienie SMOD na 0 (pojedyncza szybko tr.przez UART)
	w1_reset();								//odczyt czujnika DS1820
	w1_command(WRITE, 0);
	w1_write(-77);
	w1_command(CONVERT_T, 0);			//uruchomienie konwersji temperatury
	delay(700);								//czas na konwersj w/g materiaw firmy DALLAS
	w1_write(READ);						//odczyt temperatury,CRC jest pomijane!
	tl = w1_read();
	th = w1_read();
	
	if (w1_read() == -77) 
		return (tl | th << 8);			//funkcja zwraca odczyt lub
	else 
		return (0x8000);					//komunikat o bdzie
}

//pomiar i przesanie danych
void pomiar (void)
{
	signed int temp;
	
	temp = odczyt_DS1820();
	TRXPOWER = LEDDIODE = ON;			//zaczenie zasilania nadajnika oraz diody LED
	if (temp != 0x8000) 
		printf("%s%i\n","#",temp);		//wysanie wyniku pomiaru przez UART
	else
		printf("%s\n","#?");				//lub komunikatu o bdzie
	TRXPOWER = LEDDIODE = OFF;			//wyczenie zasilania nadajnika oraz diody LED
}

//obsuga przerwania pochodzcego od Timer'a 0
void IRQ_Timer0 (void) interrupt 1
{
	static irqcnt = 0;					//zmienna przeznaczona na licznik wej do funkcji obsugi
												//przerwania Timer'a 0
	
	if (++irqcnt > 100)					//zwikszenie stanu licznika oraz podjcie akcji, jeli > 100
	{
		TR0 = EA = DISABLE;				//wyczenie Timera 0 i przerwa
		irqcnt = 0;							//wyzerowanie licznika
		pomiar();							//wywoanie funkcji pomiaru i przesania danych
		TR0 = EA = ENABLE;				//zaczenie Timera 0 i przerwa
	}
}

