/*****************************************************************
 Odbiornik termometru radiowego zbudowany z wykorzystaniem ukadu 
 AT89C2051 oraz moduw RR i RT firmy Soyter Elektronik
 
 Transmisja docierajca z nadajnika poprzedzona jest znakiem 
 synchronizacji, to jest symbolem "#". Przerwanie Timer'a 0
 wykorzystywane jest w celu okrelenia przekroczenia limitu
 oczekiwania na transmisj.
******************************************************************/

#pragma TINY								//wybr modelu pamici: to rozmiar kodu
												//wynikowego nie moe przekroczy 2kB
												
#pragma DEFJ(TIM1_INIT=0xF8)			//timer 1 ustala prdko transmisji
												//tutaj 2400 bodw (SMOD bdzie rwny "0")

#include <reg51.h>						//doczenie definicji rejestrw uK
#include <stdio.h>						//biblioteka funkcji obsugi UART
#include "lcd4b.h"						//biblioteka funkcji obsugi wywietlacza LCD

//definicje staych symbolicznych:
#define ENABLE		1						//dla bitw rejestrw kontrolnych
#define DISABLE	0
#define ON	0								//dla diody LED
#define OFF	1


//zmienna przeznaczona na licznik wej do funkcji 
//obsugi przerwania Timer'a 0
char irqcnt = 0;

//zmienna zawierajca acuch do wywietlenia; moe to 
//by warto temperatury lub komunikat o bdzie
char * scr = "HELLO!  ";

//wymiana funkcji "putchar" w stdio.h
int putchar (const int c)
{
	LcdWrite(c);
	return (0);
}

//program gwny
void main (void)
{
	unsigned char znak;
	
	LcdInitialize();						//inicjalizacja LCD w trybie 4 bity
	LcdClrScr();							//czyszczenie ekranu LCD
	printf("%s", *scr);					//wypisanie tekstu powitania
	Delay(2000);							//pauza 2 sekundy
	LcdClrScr();							//ponowne czyszczenie ekranu LCD
	TR0 = EA = ENABLE;					//zaczenie Timera 0 i jego przerwa
	
	for (;;)									//ptla nieskoczona,w niej oczekiwanie na odbir
	{
		do 									//oczekiwanie na odbir znaku oznaczajcego
			znak = _getkey(); 			//pocztek transmisji
		while (znak == '#');
	
		Home();
		LcdWrite('*');
		scr = "";							//oczyszczenie bufora odbioru
		while (1)
		{										//jeli kod odebranych znakw wikszy od kodu 
			if (znak > 0x20) scr = scr + znak;	//spacji,to doczanie ich do acucha
			else break;						//inaczej zakoczenie pracy ptli "while"
		}
		
		irqcnt = 0;							//zerowanie licznika "timeout"
		Home();								//ustawienie si na wsp.0,0 na wywietlaczu
		printf("%c%s%s",' ',scr,"st.C");	//wywietlenie temperatury
	}
}

//obsuga przerwania pochodzcego od Timer'a 0
void IRQ_Timer0 (void) interrupt 1
{
	if (++irqcnt > 230)					//zwikszenie stanu licznika oraz podjcie akcji, jeli > 230
	{											//uwaga: nadajnik wysya dane co irqcnt>100!
	
		irqcnt = 0;							//wyzerowanie licznika
		Home();
		printf("%s","--------");		//wywielenie tekstu informujcego o bdzie transmisji
	}
}

