/*****************************************
 Obsuga transmisji szeregowej przez UART
 z wykorzystaniem przerwa.
******************************************/
#include <reg51.h>
 
#define ROZM_BUFORA_TX 32
#define ROZM_BUFORA_RX 32
#define OSCYLATOR 11059200
 
unsigned char buf_wysylki[ROZM_BUFORA_TX];
unsigned char buf_odbioru[ROZM_BUFORA_RX];
unsigned char do_wysylki, wyslano;
unsigned char wysylka_wylaczona;
unsigned char do_odbioru, odebrano;


//funkcja obsugujca przerwanie UART; using 2 oznacza, e uywany jest bank rejestrw
//R0..R7 numer 2
void UART_irq (void) interrupt 4 using 2
{
	if (RI != 0)											//fragment wykonywany, gdy do_odbioru znak
	{
		RI = 0;												//zerowanie flagi "do_odbioru"
		if ((do_odbioru+1) != odebrano) buf_odbioru[do_odbioru++] = SBUF;
																//pobranie znaku do bufora odbioru, gdy jego
	}															//rozmiar jest wystarczajcy

	if (TI != 0)											//fragment wykonywany, gdy znak do wysania
	{
		TI = 0;												//zerowanie flagi "do wysyki"
		if (do_wysylki != wyslano) 
			SBUF = buf_wysylki[wyslano++];			//jeli indeksy iloci znakw i iloci znakw do
			else wysylka_wylaczona = 1;				//wysania s rne, pobierz i wylij znak
	}
}


//obliczenie rozmiaru wolnego miejsca w buforze odbioru
unsigned char rozm_bufora_odbioru (void)
{
	return (do_odbioru - odebrano);
}

//obliczenie iloci znakw pozostajcych do wysyki
unsigned char rozm_bufora_wysylki (void)
{
	return (do_wysylki - wyslano);
}

//ustawienie prdkoci transmisji, inicjacja Timer'a 1
void UART_baudrate (unsigned int baudrate)
{
	EA = 0;         			                    	//wyczenie przerwa
	TI = 0;     											//kasowanie flagi przerwania od UART
	do_wysylki = wyslano = 0;							//nie wysano i nie do_odbioru adnych danych
	wysylka_wylaczona = 1;     						//wyczenie funkcji nadawania
	TR1 = 0;     											//zatrzymanie timer'a 1
	ET1 = 0;     											//wyczenie przerwa timera 1
	PCON |= 0x80;     									//SMOD = 1, mnonik dla kwarcu x2
	TMOD &= ~0xF0;     									//ustawienie trybu pracy timer'a 1
	TMOD |= 0x20;
	TH1 = (unsigned char)(256-(OSCYLATOR/(16L*12L*baudrate)));	//wyliczenie wartoci dla TH1
	TR1 = 1;     											//uruchomienie timer'a 1
	EA = 1;                             			//zezwolenie na przerwania
}

//inicjalizacja trybu transmisji szeregowej
void UART_inicjalizacja (void)
{
	UART_baudrate (19200);
	EA = 0;                         					//wyczenie przerwa
	do_wysylki = wyslano = 0;							//zerowanie indeksw buforw nadawania i odbioru
	wysylka_wylaczona = 1;
	do_odbioru = odebrano = 0;
	SM0 = 0; SM1 = 1; 									//ustawienie trybu pracy UART na "mode 1"
	SM2 = 0;
	REN = 1; 												//zezwolenie na prac odbiornika UART
	TI = RI = 0;											//kasowanie flag przerwania UART
	ES = 1; 													//zezwolenie na przyjmowanie przerwa od UART
	PS = 0; 													//ustawienie niskiego priorytetu dla przerwania UART
	EA = 1;                         					//zaczenie przerwa
}

//przykad wasnej implementacji funkcji wysyajcej znak przez UART
signed char _putchar(unsigned char c)
{
	if ((ROZM_BUFORA_TX-rozm_bufora_wysylki ()) <= 2) return (-1);	//bufor zbyt may, bd
	EA = 0;                         					//wyczenie przerwa
	buf_wysylki[do_wysylki++] = c;					//wstawienie znaku do bufora nadawania
	if (wysylka_wylaczona) 								//jeli nadawanie jest wyczone
	{
		wysylka_wylaczona = 0;
		TI = 1; 												//zacz je
	}
	EA = 1;                         					//zaczenie przerwa
	return (0);												//jeli operacja przebiega poprawnie, zwr 0
}

//przykad wykonania funkcji odbierajcej znak z UART
signed int _getchar (void)
{
	unsigned char c;

	if (rozm_bufora_odbioru() == 0) return (-1);	//brak odebranych znakw, bd
	EA = 0;                         					//wyczenie przerwa
	c = buf_odbioru[odebrano++];						//pobranie znaku z bufora
	EA = 1;                         					//zaczenie przerwa
	return (c);
}

void main()
{
	UART_inicjalizacja();
}
